from abc import abstractmethod

from fabric import Connection

from . import Folder,logger

class SshMan(object):

	def __init__(self,fabric_connection: Connection,remote_workdir: Folder):
		self.conn: Connection=fabric_connection
		# if isinstance(remote_workdir,str): remote_workdir=remote_projects_dir.create(
		# 	remote_workdir)
		assert isinstance(remote_workdir,Folder),f"'remote_workdir' is neither a Folder nor a string."
		self.work_folder: Folder=remote_workdir
		# if self.conn.run(f'[ -d "{remote_workdir.path}" ] && echo "exist" || echo "not exist"',warn=True).stdout.strip()=='not exist':
		# 	simple_logger.info('Creating folder %s on remote host.',remote_workdir.path)
		# 	self.conn.run(f'mkdir -p {remote_workdir.path}')
		pass

	def check_exists(self,program_name):
		which_result=self.conn.run(f'which {program_name}',warn=True)
		if which_result.failed: return False
		else:
			logger.info(f'"%s" already exists.',program_name)
			return True

	def apt_install(self,*package_names,forced=False):
		names=list()
		for package_name in package_names:
			pack_names=[x for x in package_name.split(" ") if x]
			names.extend(pack_names)

		if not forced:
			for package_name in names.copy():
				if self.check_exists(package_name):
					names.remove(package_name)
					logger.info('Skipping installation of "%s".',package_name)

		for package_name in names:
			self.run(f'sudo apt-get install {package_name} -y')
			package_path=self.conn.run(f'which {package_name}',warn=True).stdout.strip()
			logger.debug('"%s" path: %s',package_name,package_path)

	@abstractmethod
	def initial_setup(self,*args,**kwargs):
		pass

	@abstractmethod
	def execute(self,*args,**kwargs):
		pass

	def run(self,command,**kwargs):
		logger.info('Running !!!%s!!!',command)
		return self.conn.run(command,**kwargs)

	def put(self,local,remote,preserve_mode=True,absolute_remote=False):
		if not absolute_remote: remote=self.work_folder.get_filepath(remote)
		logger.info('Putting from !!!%s!!! to !!!%s!!!',local,remote)
		out=self.conn.put(local,remote=remote,preserve_mode=preserve_mode)
		return out

	def get(self,remote,local=None,preserve_mode=True):
		logger.info('Getting from !!!%s!!! to !!!%s!!!',remote,local)
		out=self.conn.get(remote,local=local,preserve_mode=preserve_mode)
		return out

	def cd(self,path):
		return self.conn.cd(path=path)

	def cd_to_working_dir(self):
		return self.cd(self.work_folder.path)

	def prefix(self,command):
		return self.conn.prefix(command=command)

	def sudo(self,command,**kwargs):
		return self.conn.sudo(command=command,**kwargs)

# class SshFolder(Folder):
#
# 	def __init__(self,remote_path,pseudo=False,parent=None,**conn_kwargs):
# 		super(SshFolder,self).__init__(remote_path,pseudo=True,parent=parent,propagate_type=True)
# 		self.pseudo=pseudo
# 		if parent is None:
# 			self.fabric_conn=Connection(**conn_kwargs)
# 		else:
# 			# self.sshman: SshMan=parent.sshman
# 			# self.sshman.work_folder=self
# 			self.fabric_conn=parent.fabric_conn
# 		self.sshman=SshMan(self.fabric_conn,self)
#
# 	def _mkdir(self,child_dirpath):
# 		with self.sshman.cd_to_working_dir():
# 			self.sshman.conn.run(f'mkdir -p {child_dirpath}')
# 		return os.path.join(self.path,child_dirpath)
#
# 	def children(self):
# 		with self.sshman.cd_to_working_dir():
# 			item_names=self.sshman.run('ls').stdout.strip().split('\n')
# 		return list(map(lambda x:os.path.join(self.path,x),item_names))
#
# 	def delete(self):
# 		with self.sshman.cd_to_working_dir():
# 			self.sshman.run(f"""
# cd ..
# rm -r {self.path}
# """)
#
# 	def clear(self):
# 		with self.sshman.cd_to_working_dir():
# 			self.sshman.run(f'rm -rf *')
#
# 	def md5(self,remote_filepath):
# 		return self.sshman.conn.run(f'md5sum {remote_filepath}',warn=True).stdout.strip().split(' ')[0]
#
# 	def upload_item(self,local_itempath,only_children=True):
# 		item_base_name=os.path.split(local_itempath)[-1]
# 		if os.path.isdir(local_itempath):
# 			if only_children:
# 				for item_to_upload in Folder(local_itempath).children():
# 					self.upload_item(item_to_upload,only_children=False)
# 				pass
# 			else:
# 				remote_folder=self.create(item_base_name)
# 				for local_sub_item_path in Folder(local_itempath).children():
# 					remote_folder.upload_item(local_sub_item_path)
# 		else:
# 			local_checksum=utils.md5(local_itempath)
# 			remote_filepath=self.get_filepath(item_base_name)
# 			while True:
# 				self.sshman.run(f"""rm {remote_filepath}""",warn=True)
# 				self.sshman.put(local_itempath,remote_filepath,absolute_remote=True)
# 				if local_checksum==self.md5(remote_filepath): return remote_filepath
#
# 	def run(self,command,**kwargs):
# 		return self.sshman.run(command=command,**kwargs).stdout.strip()
#
# 	def run_with_cd(self,command,**kwargs):
# 		with self.sshman.cd_to_working_dir():
# 			return self.run(command,**kwargs)
#
# 	def get(self,remote,local=None,preserve_mode=True):
# 		return self.sshman.get(remote,local=local,preserve_mode=preserve_mode)
#
# 	def put(self,local,remote,preserve_mode=True,absolute_remote=False):
# 		return self.sshman.put(local,remote,preserve_mode=preserve_mode,absolute_remote=absolute_remote)



# a1=shg.Polygon(np.random.rand(4,2))
# a2=shg.Polygon()
# b1=shg.MultiPolygon()
# b2=shg.MultiPolygon([a1])
# b3=shg.MultiPolygon([a1,shg.Polygon(np.random.rand(4,2),[np.random.rand(3,2)])])
# c1=SpatialGeom(a1)
# #c2=c1.convert_crs(32642)
# print(convert_crs(np.random.rand(10,2),32642))

# p=shg.Point(78,43)
	# print(reproject_point(p,4326,3857))
	# print(p.x)


import functools
import sys
from invoke.exceptions import UnexpectedExit
from lgblkb_tools import logger,Folder
from fabric import Connection,Result
from invoke import Context
from lgblkb_tools.common.proxies import RecursiveProxy

RemoteFolder:Folder=functools.partial(Folder,reactive=False)

class FabricConnection(RecursiveProxy):
	def __init__(self,host,user=None,port=None,config=None,gateway=None,forward_agent=None,
	             connect_timeout=None,connect_kwargs=None,inline_ssh_env=None,):
		wrapped=Connection(host,user=user,port=port,config=config,gateway=gateway,forward_agent=forward_agent,
		                   connect_timeout=connect_timeout,connect_kwargs=connect_kwargs,inline_ssh_env=inline_ssh_env,)
		super(FabricConnection,self).__init__(wrapped=wrapped)
	
	def inherit_out(self,out):
		return RecursiveProxy(out)
	
	@logger.trace()
	def run(self,command,raw=False,**kwargs) -> Result:
		logger.debug('command: %s',command)
		try:
			out=self.__wrapped__.run(command,**kwargs)
			# if kwargs.get('hide'): logger.debug('out:\n%s',out)
			if raw: return out
			else: return out.stdout.strip()
		except UnexpectedExit as exc:
			# logger.debug('dir(exc): %s',dir(exc))
			logger.error('exc: %s',exc)
			sys.exit(-1)
	
	def cd(self,path):
		if isinstance(path,Folder): path=path.path
		return self.__wrapped__.cd(path)

class Remote(object):
	def __init__(self,connection):
		self.conn: Connection=connection
		self.folder: Folder=RemoteFolder(self.conn.run('pwd'))
	
	def create_file(self,filepath,file_contents):
		file_contents=file_contents.replace('"','""')
		self.conn.run(f'sudo bash -c "cat <<EOF > {filepath}\n{file_contents}\nEOF"')
		return self
	
	def backup(self,filepath):
		self.conn.run(f'sudo cp {filepath} {filepath}.bak')
		return self
	
	def replace_in_file(self,filepath,old,new):
		out=self.conn.run(f'cat {filepath}',raw=True).stdout
		avahi_modified=out.replace(old,new)
		self.create_file(filepath,avahi_modified)
		return self
	
	@logger.trace()
	def share_wifi_over_lan(self,first_time=False):
		netplan_folder: Folder=RemoteFolder('/etc/netplan')
		with self.conn.cd(netplan_folder):
			filenames=self.conn.run(f'ls').split('\n')
			yaml_file=[filename for filename in filenames if filename.endswith('.yaml')][0]
			logger.debug('yaml_file: %s',yaml_file)
			yaml_contents=f"""
# This file is generated from information provided by
# the datasource.  Changes to it will not persist across an instance.
# To disable cloud-init's network configuration capabilities, write a file
# /etc/cloud/cloud.cfg.d/99-disable-network-config.cfg with the following:
# network: {{config: disabled}}
network:
    ethernets:
        eth0:
            dhcp4: true
            optional: true
    version: 2
    wifis:
        wlan0:
            optional: true
            access-points:
                NU:
                    password: 1234512345
            dhcp4: true
"""
		# self.create_file(yaml_file,yaml_contents)
		# self.conn.run('sudo netplan --debug try')
		# self.conn.run('sudo netplan --debug generate')
		# self.conn.run('sudo netplan --debug apply')
		crda_filepath='/etc/default/crda'
		# self.conn.run(f'sudo touch {crda_filepath}')
		crda_content="""
# Define your country code here as explained above or in this format if not explained:
# ISO/IEC 3166-1 alpha2 country code
REGDOMAIN=KZ
"""
		# self.create_file(crda_filepath,crda_content)
		# self.conn.run(f'cat {crda_filepath}')
		# self.conn.run(f'sudo apt-get update && sudo apt-get dist-upgrade')
		# self.conn.run(f'sudo apt-get install wireless-tools wpasupplicant openssh-server')
		# self.conn.run(f'sudo apt-get install parprouted dhcp-helper avahi-daemon')
		dhcp_relay_file='/etc/default/dhcp-helper'
		dhcp_contents="""
# relay dhcp requests as broadcast to wlan0
DHCPHELPER_OPTS="-b wlan0"
"""
		if first_time: self.backup(dhcp_relay_file)
		# self.create_file(dhcp_relay_file,dhcp_contents)
		# self.conn.run(f'cat {dhcp_relay_file}')
		avahi_conf_file=r'/etc/avahi/avahi-daemon.conf'
		if first_time: self.backup(avahi_conf_file)
		# self.replace_in_file(avahi_conf_file,'[reflector]',
		#                      """
		# 					 [reflector]
		# 					 enable-reflector=yes""")
		arp_bridge_service_file=r'/lib/systemd/system/arp-bridge.service'
		arp_bridge_service_content="""
[Unit]
Description=ARP Bridge over Wireless Interface
Wants=network-online.target
After=network-online.target

[Service]
Type=idle
RemainAfterExit=yes
ExecStart=/lib/systemd/system/set-arp-routing

[Install]
WantedBy=multi-user.target
"""
		# self.create_file(arp_bridge_service_file,arp_bridge_service_content)
		starting_script_path=r'/lib/systemd/system/set-arp-routing'
		starting_script_content="""
#!/bin/bash

## Start Process
ETHERNET_IFACE=eth0
WIRELESS_IFACE=wlan0

## Setup system forwarding
echo "Enable IP forwarding"
echo 1 > /proc/sys/net/ipv4/ip_forward
## Uncomment lines below if you don't want to use parprouted.
#echo "Enable ARP forwarding"
# Comment for all interfaces
#echo 1 > /proc/sys/net/ipv4/conf/${WIRELESS_IFACE}/proxy_arp
# Uncomment for all interfaces
#echo 1 > /proc/sys/net/ipv4/conf/all/proxy_arp

## Fix the 'eth0' interface that don't want to mount at boot on 3b+
## Uncomment this part if the 'eth0' interface is not mounted at boot
#/usr/sbin/netplan apply

## A little sleep
sleep 5

## Assign address
echo "Cloning IP from ${WIRELESS_IFACE} to ${ETHERNET_IFACE}"
/sbin/ip addr add $(/sbin/ip addr show $WIRELESS_IFACE | perl -wne 'm|^\s+inet (.*)/| && print $1')/32 dev $ETHERNET_IFACE

## Uncomment lines below in case you're encountering the same issues
#echo "Removing bad APIPA IP from ${ETHERNET_IFACE}"
#/sbin/ip addr del $(/sbin/ip addr show $ETHERNET_IFACE | perl -wne 'm|^\s+inet (169.254.*)/| && print $1')/16 dev $ETHERNET_IFACE
#echo "Removing bad APIPA route from ${ETHERNET_IFACE}"
#/sbin/ip route del 169.254.0.0/16 dev $ETHERNET_IFACE

## Make sure that the eth0 interface is up
echo "Setting up lan interface"
/sbin/ip link set $ETHERNET_IFACE up

## Setup ARP forwarding
echo "Starting paraprouted"
/usr/bin/killall -KILL parprouted 2> /dev/null
/usr/sbin/parprouted $ETHERNET_IFACE $WIRELESS_IFACE

## Reloading DHCP Relay
echo "Start / Reload DHCP Relay"
/bin/systemctl restart dhcp-helper

## A little sleep
sleep 5

## Refresh local ARP cache
/sbin/ip -s -s neigh flush all

## Disable wireless power management
## Uncomment if your wireless speed is too low
#/sbin/iwconfig $WIRELESS_IFACE power off

## Not related to network at all but useful for me,
## as my keyboard does not have an English layout.
## Uncomment if you find it useful too.
#loadkeys ch

## End Process
"""
		# self.create_file(starting_script_path,starting_script_content)
		# self.conn.run(f'sudo chmod -v u+x {starting_script_path}')
		self.conn.run("""
sudo systemctl daemon-reload
sudo systemctl enable arp-bridge.service
sudo systemctl start arp-bridge.service
systemctl status wpa_supplicant.service arp-bridge.service dhcp-helper.service
""")
		
		pass

def main():
	connection=FabricConnection('10.1.143.179',user='ubuntu')
	remote=Remote(connection)
	remote.share_wifi_over_lan()
	
	pass

if __name__=='__main__':
	main()

# msg="Ran {0.command!r} on {0.connection.host}, got stdout:\n{0.stdout}"
# print(msg.format(result))



# dat_src,dlayer=DataSet.from_array(digitized,ndvi_ds).polygonize('Memory','',
#                                                                 field_name=field_name)
# poly_collection=cleanup_polygons(dlayer,field_name,area_threshold=110,thresh_len=8)
# if 0 in poly_collection: poly_collection.pop(0)
# dat_src=dlayer=None
# #print(list(poly_collection.keys()))
# features=list()
# for cat_id,geoms in poly_collection.items():
# 	simple_logger.debug('cat_id: %s',cat_id)
# 	feat=geojson.Feature(id=cat_id,geometry=shg.MultiPolygon(geoms),
# 	                     properties=dict(DN=float(mean_vals[cat_id])))
# 	features.append(feat)
# feat_col=geojson.FeatureCollection(features,cad_num=spk.cad_land.name)
# geojson.dump(feat_col,open('arkan5_simplified_12.geojson','w'))



# print(get_group_cloudiness())
	# cad_info: Cadastres_Info=db_opers.SessionHelper(db_opers.Cadastres_Info).get_with_id(1380560,as_saved=True)
	# band_paths=list()
	# band_paths.append(
	# 	r'/home/lgblkb/PycharmProjects/Egistic/Downloaded_images/sentinel2/unzipped_scenes/S2B_MSIL2A_20180518T062629_N0206_R077_T42UUB_20180518T101322.SAFE/GRANULE/L2A_T42UUB_A006252_20180518T063426/IMG_DATA/R10m/T42UUB_20180518T062629_B04_10m.jp2')
	# band_paths.append(
	# 	r'/home/lgblkb/PycharmProjects/Egistic/Downloaded_images/sentinel2/unzipped_scenes/S2B_MSIL2A_20180518T062629_N0206_R077_T42UUB_20180518T101322.SAFE/GRANULE/L2A_T42UUB_A006252_20180518T063426/IMG_DATA/R10m/T42UUB_20180518T062629_B03_10m.jp2')
	# band_paths.append(
	# 	r'/home/lgblkb/PycharmProjects/Egistic/Downloaded_images/sentinel2/unzipped_scenes/S2B_MSIL2A_20180518T062629_N0206_R077_T42UUB_20180518T101322.SAFE/GRANULE/L2A_T42UUB_A006252_20180518T063426/IMG_DATA/R10m/T42UUB_20180518T062629_B02_10m.jp2')
	# no_data_value=0
	# p1=gsup.results_folder.create('test1').get_filepath('ndvi',ext='.tiff',include_datetime=True)
	# p2=gsup.results_folder.create('test1').get_filepath('ndvi_merged',ext='.tiff',include_datetime=True)
	# rgb_to_geotiff(p1,cutline_json=geojson.Feature(geometry=db_opers.to_shape(cad_info.geom)),*band_paths,no_data_value=no_data_value)
	# gsup.run_shell(gsup.gdal_merge_py,'-o',p2,'-n',str(no_data_value),'-a_nodata','nan',p1,with_popen=True)

	# np.random.seed(1)

	# val_ranges=[-1,0,0.033,0.066,0.1,0.133,0.166,0.2,0.25,0.3,0.35,0.4,0.45,0.5,0.6,0.7,0.8,0.9,1]
	# simple_logger.debug('len(val_ranges): %s',len(val_ranges))

	# zero_catch_window=1e-9
	# val_ranges=[-1.01,-zero_catch_window,zero_catch_window,*np.linspace(zero_catch_window,1,12)]
	# mean_vals,digitized=categorize(ndvi_ds.array,val_ranges)
	# digitized*=digitized!=2
	# mean_vals.pop(2)
	# print(mean_vals)
	# print(len(mean_vals))
	# #digitized=remove_lonely_cats(digitized)
	# field_name='Category'
	# #DataSet.from_array(digitized,ndvi_ds).polygonize('geojson','arkan5_original_12.geojson',field_name=field_name)
	# contour_dsrc,contour_layer=ndvi_ds.generate_contours(val_ranges,'Memory','')
	# feats=process_vector_layer(contour_layer,100)
	# contour_dsrc=contour_layer=None
	# save_json_features(feats,'reduced_13',cad_num=cad_num)


import collections
import hashlib
import multiprocessing as mp
import functools
import os
import signal
import subprocess
import sys
import time
from typing import Sized

import dateutil
from datetime import datetime,timedelta,date

from .. import logger

def reprer(obj):
	if isinstance(obj,dict): d=obj
	else: d=obj.__dict__
	body=[f"{k:<20}: {('None' if v is None else v)}" for k,v in d.items() if k[0]!='_']
	if body: delim="\n"
	# else: body,delim=["Expired!"]," "
	else: body,delim=['Empty']," "
	return f'"{obj.__class__.__name__}" item:'+delim+"\n".join(body)

def func_has_classarg(func,args):
	has_classarg=False
	if args:
		funcattr=getattr(args[0],func.__name__,None)
		if funcattr is not None and hasattr(funcattr,'__self__'):
			has_classarg=True
	return has_classarg

def infiterate(iter_obj,max_iter_count=None,next_getter=None,inform_count=8,on_inform=None):
	if on_inform is None: on_inform=lambda *args,**kwargs:True
	if isinstance(iter_obj,int):
		an_iterable,iter_count=range(iter_obj),iter_obj
	else:
		an_iterable=iter_obj
		if isinstance(iter_obj,Sized):
			iter_count=len(iter_obj)
		else:
			assert max_iter_count
			iter_count=max_iter_count
	max_iter_count=max_iter_count or iter_count
	if next_getter is None: next_getter=lambda _obj:_obj
	for i,obj in enumerate(an_iterable):
		if i%(max_iter_count//inform_count)==0:
			on_inform()
			logger.info('%d%%, i=%d',i/(max_iter_count-1)*100,i)

		obj=next_getter(obj)
		yield obj
		if i==max_iter_count-1:
			on_inform()
			logger.info('%d%%, i=%d',i/(max_iter_count-1)*100,i)
			return

class ParallelTasker:

	def __init__(self,func,*args,**kwargs):
		self.partial_func=functools.partial(func,*args,**kwargs)
		self.queue=mp.Queue()
		self._total_proc_count=0
		pass

	def set_run_params(self,**kwargs):
		val_lengths={len(v) for v in kwargs.values()}
		assert len(val_lengths)==1
		val_length=val_lengths.pop()

		for i in range(val_length):
			self.queue.put((i,{k:v[i] for k,v in kwargs.items()}))
			self._total_proc_count+=1
		# simple_logger.info('self._total_proc_count: %s',self._total_proc_count)

		return self

	def __process_func(self,queue,common_list):
		i,kwargs=queue.get()
		result=self.partial_func(**kwargs)
		common_list.append([i,result])

	@staticmethod
	def __join_finished_processes(active_procs,sleep_time):
		while True:
			# simple_logger.info('Process queue is full. Searching for finished processes.')
			for p in active_procs:
				if not p.is_alive():
					# simple_logger.info('Finished process found. Joining it.')
					active_procs.remove(p)
					p.join()
					# p.terminate()
					# simple_logger.info('Process successfully removed.')
					return
			time.sleep(sleep_time)

	def run(self,workers_count=None,sleep_time=1.0):
		workers_count=min(mp.cpu_count()-1,workers_count or self._total_proc_count)
		manager=mp.Manager()
		common_list=manager.list()
		processes=[mp.Process(target=self.__process_func,args=(self.queue,common_list)) for _ in range(self.queue.qsize())]
		active_procs=list()
		while True:
			# simple_logger.info('loop begins')
			if len(processes)==0:
				# simple_logger.info('Waiting for the last jobs to finish.')
				for active_p in active_procs:
					active_p.join()
				break
			if len(active_procs)<workers_count:
				# simple_logger.info('Adding a process')
				proc=processes.pop()
				proc.start()
				active_procs.append(proc)
			else:
				self.__join_finished_processes(active_procs,sleep_time)
			time.sleep(sleep_time)
		return [item[1] for item in sorted(common_list,key=lambda x:x[0])]

def run_shell(*args,non_blocking=False,chdir=None,with_popen=False,ret_triggers=None,**kwargs):
	chdir=chdir or os.getcwd()
	normal_dir=os.getcwd()
	os.chdir(chdir)

	if non_blocking:
		subprocess.Popen(args,stdout=subprocess.PIPE,**kwargs)
	else:
		#output=subprocess.run(args,stdout=subprocess.PIPE,**kwargs).stdout.decode()
		#output=subprocess.run(args,stdout=subprocess.PIPE,**kwargs).stdout.decode()
		if with_popen:
			process=subprocess.Popen(args,stdout=subprocess.PIPE,stderr=subprocess.STDOUT,preexec_fn=os.setsid)
			regular_termination=True
			nextline=''
			while regular_termination:
				nextline=process.stdout.readline().decode()
				# print('nextline:',nextline)
				if nextline=='' and process.poll() is not None:
					break
				if ret_triggers:
					for ret_trigger in ret_triggers:
						if ret_trigger in nextline:
							regular_termination=False
							break
				logger.debug(nextline)
				# sys.stdout.write(nextline)
				sys.stdout.flush()
			if regular_termination:
				output=process.communicate()[0]
				exitCode=process.returncode
				if exitCode!=0:
					raise Exception(args,exitCode,output)
			else:
				logger.info(r'Manual process termination triggered with line:\n%s',nextline)
				logger.debug('Killing process: %s',process.pid)
				os.killpg(os.getpgid(process.pid),signal.SIGTERM)
				raise Exception('Manual process termination.')
		else:
			subprocess.run(args,stdout=subprocess.PIPE,**kwargs).stdout.decode()
	os.chdir(normal_dir)

def period_within(days=None,start_date=None,end_date=None):
	if isinstance(start_date,str): start_date=dateutil.parser.parse(start_date)
	if isinstance(end_date,str): end_date=dateutil.parser.parse(end_date)
	if days is not None:
		if start_date is not None:
			end_date=start_date+timedelta(days=days)
		elif end_date is not None:
			start_date=end_date-timedelta(days=days)
		else:
			end_date=date.today()
			start_date=end_date-timedelta(days=days)
	else:
		end_date=end_date or date.today()
	return start_date,end_date

def datetime_within(start=None,end=None,**timedelta_opts):
	if timedelta_opts:
		if start is not None:
			end=start+timedelta(**timedelta_opts)
		elif end is not None:
			start=end-timedelta(**timedelta_opts)
		else:
			end=datetime.now()
			start=end-timedelta(**timedelta_opts)
	else:
		end=end or datetime.now()
	return start,end

def md5(filepath):
	hash_md5=hashlib.md5()
	with open(filepath,"rb") as f:
		for chunk in iter(lambda:f.read(4096),b""):
			hash_md5.update(chunk)
	return hash_md5.hexdigest()

def get_md5(text):
	return hashlib.md5(str(text).encode('utf-8')).hexdigest()

def run_command(cmd):
	logger.debug('cmd:\n%s',cmd)
	process=subprocess.Popen(cmd,stdout=subprocess.PIPE,shell=True)
	for c in iter(lambda:process.stdout.read(1),b''):  # replace '' with b'' for Python 3
		sys.stdout.write(c.decode(sys.stdout.encoding,errors='ignore'))

def dict_merge(dct,merge_dct,add_keys=True):
	""" Recursive dict merge. Inspired by :meth:``dict.update()``, instead of
	updating only top-level keys, dict_merge recurses down into dicts nested
	to an arbitrary depth, updating keys. The ``merge_dct`` is merged into
	``dct``.
	:param add_keys: add missing keys from merge_dct to dct.
	:param dct: dict onto which the merge is executed
	:param merge_dct: dct merged into dct
	:return: None
	"""
	# for k,v in merge_dct.iteritems():
	# 	if (k in dct and isinstance(dct[k],dict)
	# 			and isinstance(merge_dct[k],collections.Mapping)):
	# 		dict_merge(dct[k],merge_dct[k])
	# 	else:
	# 		dct[k]=merge_dct[k]

	for k,v in merge_dct.items():
		if isinstance(dct.get(k),dict) and isinstance(v,collections.Mapping):
			dct[k]=dict_merge(dct[k],v,add_keys=add_keys)
		else:
			dct[k]=v
	return dct



from lgblkb_tools import logger,Folder
import pandas as pd

def main():
	filepath=r'/home/lgblkb/Desktop/База население.xlsx'
	filepath=r'/home/lgblkb/Desktop/База предприниматели.xlsx'
	df=pd.read_excel(filepath)
	logger.debug('df:\n%s',df)
	
	pass

if __name__=='__main__':
	main()



import wrapt
import functools
import glob
import os
import shutil
import zipfile
from datetime import datetime
from typing import Callable
from box import Box
from checksumdir import dirhash
from . import logger

get_name=lambda some_path:os.path.splitext(os.path.basename(some_path))[0]

def get_parent_dir(some_path,depth=1):
	for i in range(depth):
		some_path=os.path.dirname(os.path.abspath(some_path))
	return some_path

def create_path(*paths,stop_depth=0):
	path=os.path.join(*paths)
	os.makedirs(get_parent_dir(path,stop_depth),exist_ok=True)
	return path

def get_existing_path(paths,safe=False):
	for p in paths:
		if os.path.exists(p): return p
	if not safe:
		raise FileNotFoundError('Could not find any existing path.')

def get_splitted(path):
	folders=[]
	while 1:
		path,folder=os.path.split(path)
		
		if folder!="":
			folders.append(folder)
		else:
			if path!="":
				folders.append(path)
			break
	folders.reverse()
	return folders

def create_zipfile(dir_to_zip,savepath=''):
	"""Create a zip file from all the files under 'dir_to_zip'.

	The output zip file will be saved to savepath.
	If savepath ends with '.zip', then the output zip file will be
	saved AS 'savepath'. Necessary tree subdirectories are created automatically.
	Else, savepath is assumed to be a directory path,
	hence the output zip file will be saved TO 'savepath'
	directory. Necessary tree subdirectories are created automatically.

	:return absolute savepath
	"""
	save_cwd=os.getcwd()
	dir_to_zip=os.path.abspath(dir_to_zip)
	if dir_to_zip in os.path.split(savepath)[0]: raise ValueError(
		'To avoid recursion), resultant "savepath" should not be located inside "dir_to_zip"',
		dict(dir_to_zip=dir_to_zip,savepath=savepath))
	parent_dir,dir_name=os.path.split(dir_to_zip)
	os.chdir(parent_dir)
	if savepath:
		if savepath.endswith('.zip'):
			create_path(savepath,stop_depth=1)
		else:
			create_path(savepath,stop_depth=0)
			savepath=os.path.join(savepath,dir_name+'.zip')
	else: savepath=dir_to_zip+'.zip'
	
	pwd_length=len(os.getcwd())
	with zipfile.ZipFile(savepath,"w",compression=zipfile.ZIP_DEFLATED) as zf:
		for dirpath,dirnames,filenames in os.walk(dir_to_zip):
			for name in sorted(dirnames):
				path=os.path.normpath(os.path.join(dirpath,name))
				zf.write(path,path[pwd_length+1:])
			
			for name in filenames:
				path=os.path.normpath(os.path.join(dirpath,name))
				if os.path.isfile(path):
					zf.write(path,path[pwd_length+1:])
	
	os.chdir(save_cwd)
	return os.path.abspath(savepath)

def generate_name_portions(*args,__kv_delim__='-',**kwargs):
	arg_parts=[str(x) for x in args]
	kwarg_parts=[f"{k}{__kv_delim__}{v}" for k,v in kwargs.items() if v is not None]
	return arg_parts+kwarg_parts

def _get_name_parts_from_depth(path,include_depth):
	if type(include_depth) is Callable:
		parent_parts=include_depth(get_splitted(path))
	elif type(include_depth) is int:
		parent_parts=get_splitted(path)[-include_depth:]
	elif type(include_depth) in [tuple,list]:
		parent_parts=[get_splitted(path)[-x] for x in include_depth]
	elif type(include_depth) is slice:
		parent_parts=get_splitted(path)[-include_depth]
	else:
		raise ValueError(f'include_depth={include_depth}')
	return parent_parts

class Folder(os.PathLike):
	
	def __init__(self,path='',reactive=True,assert_exists=False):
		if isinstance(path,Folder): path=path.path
		path=os.path.abspath(os.path.normpath(path))
		if self._is_file(path): path=get_parent_dir(path)
		if not self._exists(path):
			# possible_extension=os.path.splitext(path)[-1]
			# if possible_extension:
			# 	logger.warning('Provided folder path (which does not exist) appears to have an extension %s',possible_extension)
			# 	pass
			if assert_exists: raise AssertionError('Path should already exist.',dict(path=path))
			elif reactive: create_path(path)
			
		
		self.path=path
		self._class=functools.partial(self.__class__)
		self._inherit_class=self._class
		self.reactive=reactive
	
	@property
	def name(self) -> str:
		return os.path.split(self.path)[-1]
	
	def __str__(self):
		# return f"{self.__class__.__name__}(r'{self.path}')"
		return self.path
	
	def get_filepath(self,*name_args,ext='',delim='_',include_depth=None,datetime_loc_index=None,iterated=False,**name_kwargs):
		parts=list()
		if include_depth: parts.extend(_get_name_parts_from_depth(self.path,include_depth))
		parts.extend(generate_name_portions(*name_args,**name_kwargs))
		if datetime_loc_index is not None: parts.insert(datetime_loc_index,datetime.now().strftime("%Y-%m-%d_%H-%M-%S"))
		assert parts,'Nothing is provided to create filepath.'
		
		output_path=os.path.join(self.path,delim.join(map(str,parts)).replace(' ',delim)+ext)
		if iterated:
			return create_iterated_path(output_path)
		else:
			return output_path
	
	def _exists(self,path):
		return os.path.exists(path)
	
	def _is_file(self,path):
		return os.path.isfile(path)
	
	def _create_dir(self,dirpath,**kwargs):
		return self._inherit_class(dirpath,**kwargs)
	
	def create(self,*name_args,delim='_',datetime_loc_index=None,iterated=False,reactive=None,**name_kwargs):
		"""
		:param name_args: args that are converted to name portions.
		:param delim: delimiter to use between the name string portions
		:param datetime_loc_index: index location of datetime in the resultant list of name portions.
		:param iterated: if iterated==True, method it will first check for existence of the folder. If exists, it creates a folder with new name,
		which is generated by incrementing the folder name.
		:param reactive: if not provided, default value is used. If provided, new folder will be created with specified value.
		:param name_kwargs: kwargs that are converted to name portions.
		:return: new instance of the class.
		:rtype: Folder
		"""
		new_dirpath=self.get_filepath(*name_args,ext='',delim=delim,include_depth=None,
		                              datetime_loc_index=datetime_loc_index,iterated=iterated,**name_kwargs)
		# new_folder=self._create_dir(new_dirpath,**({} if reactive is None else dict(reactive=reactive)))
		new_folder=self._create_dir(new_dirpath,**dict(reactive=self.reactive if reactive is None else reactive))
		return new_folder
	
	def delete(self):
		# assert self.reactive,f'Folder {self.path} is not reactive'
		shutil.rmtree(self.path,ignore_errors=True)
	
	@logger.trace(skimpy=True)
	def clear(self):
		# assert self.reactive,f'Folder {self.path} is not reactive'
		self.delete()
		# try:
		#
		# except FileNotFoundError:
		# 	# lu.simple_logger.warning('FileNotFoundError when trying to clear up the results folder. Passing on.')
		# 	pass
		create_path(self.path)
		return self
	
	@property
	def children(self):
		return self.glob_search('*')
	
	def glob_search(self,*patterns,recursive=True):
		return glob.glob(self.get_filepath(*patterns),recursive=recursive)
	
	@logger.trace(skimpy=True)
	def zip(self,zip_filepath='',forced=False):
		zip_filepath=zip_filepath or self.parent().get_filepath(self.name,ext='.zip')
		if len(get_splitted(zip_filepath))==1: zip_filepath=self.parent().get_filepath(zip_filepath)
		if not zip_filepath.endswith('.zip'): zip_filepath+='.zip'
		# if not self.reactive: return zip_filepath
		
		# assert self.reactive,f'Folder {self.path} is not reactive'
		
		if os.path.exists(zip_filepath):
			if forced: os.remove(zip_filepath)
			else: return zip_filepath
		
		create_zipfile(self.path,zip_filepath)
		return zip_filepath
	
	@logger.trace(skimpy=True)
	def unzip(self,zip_filepath,create_subdir=True):
		# assert self.reactive,f'Folder {self.path} is not reactive'
		# if save_path_formatter is None: save_path_formatter=lambda x:x
		# zip_path=[zip_filepath,
		#           zip_filepath+'.zip',
		#           self.get_filepath(zip_filepath),
		#           self.get_filepath(zip_filepath,ext='.zip')]
		# zip_path=get_existing_path(zip_path)
		# assert not os.path.isdir(zip_path),f'The folder "{zip_filepath}" cannot be unzipped.'
		if create_subdir:
			zipfilename=os.path.splitext(zip_filepath)[0]
			dst_folder_path=self.create(zipfilename).path
		else:
			dst_folder_path=self.path
		if not self.reactive: return self._inherit_class(dst_folder_path)
		shutil.unpack_archive(zip_filepath,dst_folder_path,'zip')
		return self._inherit_class(dst_folder_path)
	
	@logger.trace(skimpy=True)
	def rename(self,new_name):
		new_path=self.parent().get_filepath(new_name)
		# new_path=os.path.join(get_parent_dir(self.path),new_name)
		if self.reactive: os.rename(self.path,new_path)
		self.path=new_path
		return self
	
	@logger.trace(skimpy=True)
	def move_to(self,dst_path):
		if not isinstance(dst_path,str): dst_path=dst_path.path
		# if self.reactive: shutil.move(self.path,dst_path)
		shutil.move(self.path,dst_path)
		self.path=os.path.join(dst_path,self.name)
		return self
	
	def parent(self,depth=1):
		"""

		:rtype: Folder
		"""
		return self._inherit_class(get_parent_dir(self.path,depth=depth))
	
	@logger.trace(skimpy=True)
	def copy_to(self,dst_folder,new_name='',forced=False):
		"""

		:rtype: Folder
		"""
		dst_folder_path=self._inherit_class(dst_folder).get_filepath(new_name or self.name)
		delete_dst=lambda:shutil.rmtree(dst_folder_path,ignore_errors=True)
		resulter=lambda:self._class(dst_folder_path)
		
		if os.path.exists(dst_folder_path):
			if forced: delete_dst()
			else:
				source_hashsum=dirhash(self.path)
				destination_hashsum=dirhash(dst_folder_path)
				if source_hashsum==destination_hashsum: return resulter()
				else: delete_dst()
		
		shutil.copytree(self.path,dst_folder_path)
		return resulter()
	
	def size(self,byte_factor=1e-9):
		total_size=0
		for dirpath,dirnames,filenames in os.walk(self.path):
			for f in filenames:
				fp=os.path.join(dirpath,f)
				total_size+=os.path.getsize(fp)
		return total_size*byte_factor
	
	@property
	def exists(self):
		return self._exists(self.path)
	
	# def create_iterated(self,start=None,delim='_',empty_ok=False):
	# 	if empty_ok:
	# 		validator_func=lambda existing_path:not self._inherit_class(existing_path).children
	# 	else:
	# 		validator_func=lambda existing_path:existing_path==self.path
	#
	# 	new_iter_folder_path=create_iterated_path(self.path,start,delim,validator_func=validator_func)
	# 	last_part=new_iter_folder_path.split(delim)[-1]
	# 	if last_part.isnumeric() and int(last_part)==(start or 0):
	# 		new_folder=self.rename(new_iter_folder_path)
	# 	else:
	# 		new_folder=self.__class__(new_iter_folder_path)
	# 		if not new_folder==self and not self.children(): self.delete()
	# 	return new_folder
	
	def __getitem__(self,item):
		item=str(item)
		if os.path.splitext(item)[-1]:
			return self.get_filepath(item)
		else:
			return self.create(item)
	
	def __eq__(self,other):
		if isinstance(other,str):
			return self.path==other
		else:
			return self.path==other.path
	
	def __setitem__(self,key,value):
		filename,ext=os.path.splitext(key)
		if isinstance(value,str):
			string_obj=value
		elif type(value) in [list,tuple]:
			string_obj='\n'.join(map(str,value))
		elif isinstance(value,dict):
			ext=ext or '.yaml'
			if ext in ['.yaml','.yml']:
				Box(value).to_yaml(filename=self.get_filepath(filename,ext=ext))
				return
			elif ext=='.json':
				Box(value).to_json(filename=self.get_filepath(filename,ext=ext))
				return
			else:
				string_obj='\n'.join(map(lambda kv:f"{kv[0]}: {kv[1]}",value.items()))
		
		else:
			string_obj=str(value)
		# def writer(filehandler):
		# 	for k,v in value.items():
		# 		filehandler.writelines([f"{k}: {v}"])
		
		with open(self.get_filepath(filename,ext=ext or '.txt'),'w') as fh:
			fh.write(str(string_obj))
	
	def __repr__(self):
		return f"{self.__class__.__name__}(r'{self.path}')"
	
	# return self.path
	
	def __fspath__(self):
		return self.path

def create_iterated_path(iterated_path: str,start=None,delim='_',validator_func=None) -> str:
	if validator_func is None: validator_func=lambda x:False
	base_path,item_name_with_ext=os.path.split(iterated_path)
	item_name,ext=os.path.splitext(item_name_with_ext)
	
	curr_iter=0 if start is None else start
	while True:
		new_item_name=delim.join([item_name,f'{curr_iter}'])
		new_itempath=os.path.join(base_path,new_item_name+ext)
		if not os.path.exists(new_itempath) or validator_func(new_itempath): return new_itempath
		curr_iter+=1



import setuptools
import wrapt
from box import Box
from lgblkb_tools import Folder

with open("README.md","r") as fh:
	long_description=fh.read()

# root_folder=Path(__file__).parent
root_folder=Folder(__file__,reactive=False)

install_requires=[
	'pandas',
	'numpy',
	'python-box',
	'more-itertools',
	'sklearn',
	'scikit-image',
	'scikit_learn',
	'python_dateutil',
	'ruamel.yaml',
	'python_log_indenter',
	'colorlog',
	'requests',
	'docopt',
	'python-dotenv',
	'wrapt',

	'shapely',
	'pyproj',
	'geojson',
	'geopandas',
	'geojsonio',

	'matplotlib',

	'redis',
	'celery',
	'celery-singleton',
	'docker',
	'python-telegram-bot',
	'sqlalchemy',
	'geoalchemy2',
	'psycopg2',
	'jsonpickle',
	'fabric',
	# 'circus',
	'checksumdir',
	'joblib',

]

def setup(version):
	setuptools.setup(
		name="lgblkb_tools",
		version=version,
		author="Dias Bakhtiyarov",
		author_email="dbakhtiyarov@nu.edu.kz",
		description="Some useful tools for everyday routine coding improvisation)",
		long_description=long_description,
		long_description_content_type="text/markdown",
		url="https://github.com/lgblkb/lgblkb_tools",
		packages=setuptools.find_packages(),
		classifiers=(
			"Programming Language :: Python :: 3.6",
			"License :: OSI Approved :: MIT License",
			"Operating System :: OS Independent",
		),
		install_requires=install_requires
	)

# noinspection PyAbstractClass
class BoxProxy(wrapt.ObjectProxy):
	def __init__(self,wrapped):
		super(BoxProxy,self).__init__(wrapped=wrapped)
		self._self_new_version=None
		self._self_filepath=None
		pass

	def elevate(self,major,minor,maintenance=None,tag=''):
		print('Current version: ',self.version)
		# logger.debug('current_version: %s',self.version)
		version_portions,old_tag=self.get_version_portions_and_tag()
		prev_major,prev_minor,prev_maint=version_portions
		if major!=prev_major or minor!=prev_minor: maintenance=maintenance or 0
		elif maintenance is None: maintenance=version_portions[-1]+1
		new_version=f"{major}.{minor}.{maintenance}"
		if tag: new_version+=f'.{tag}'
		# logger.debug('new_version: %s',new_version)
		print('new_version: ',new_version)

		self._self_new_version=new_version
		return new_version

	def get_version_portions_and_tag(self):
		portions=self.version.split('.',maxsplit=3)
		if len(portions)==4: num_version,tag=portions[:3],portions[-1]
		elif len(portions)==3: num_version,tag=portions,''
		else: raise NotImplementedError('Version format is undefined.',dict(portions=portions))
		num_portions=[int(x) for x in num_version]
		return num_portions,tag

	def from_yaml(self,filepath,**kwargs):
		data=self.__wrapped__.from_yaml(filename=filepath,**kwargs)
		self._self_filepath=filepath
		out=BoxProxy(data)
		out._self_filepath=filepath
		out._self_new_version=data.version
		return out

	def update(self):
		self.__wrapped__.version=self._self_new_version
		self.to_yaml(filename=self._self_filepath)
		# root_folder[os.path.split(self.filepath)[-1]]=dict(version=self.new_version)
		return self

Elevator=BoxProxy(Box)

def get_package_info():
	return Elevator.from_yaml(root_folder['package_info.yaml'])

def main():
	build_folder=root_folder['build']
	dist_folder=root_folder['dist']
	info=get_package_info()
	build_folder.delete()
	dist_folder.delete()
	new_version=info.elevate(major=1,minor=0)
	setup(new_version)
	info.update()
	pass

if __name__=='__main__':
	main()
	pass



from contextlib import contextmanager

import diskcache
from fabric.runners import Result
from typing import Union

from lgblkb_tools.common.proxies import RecursiveProxy
from invoke import Responder

from lgblkb_tools import logger,Folder
from fabric import Connection

cache=diskcache.Cache(Folder('lgblkb_cache').path)

# noinspection PyAbstractClass
class RemoteConnection(RecursiveProxy):
	
	def _proxy_setup(self,*args,**kwargs):
		self.res: Result=None
		self.hide=False
		self.backup_tag='_lgblkb_backup'
	
	@classmethod
	def using(cls,host,user=None,port=None,config=None,gateway=None,forward_agent=None,
	          connect_timeout=None,connect_kwargs=None,inline_ssh_env=None,):
		return cls(_get_remcon(host,user=user,port=port,config=config,gateway=gateway,forward_agent=forward_agent,
		                       connect_timeout=connect_timeout,connect_kwargs=connect_kwargs,inline_ssh_env=inline_ssh_env,))
	
	@logger.trace()
	def apt_install(self,command,**kwargs):
		return self.run('sudo apt-get install -y --no-install-recommends '+command,**kwargs)
	
	def run(self,command,**kwargs):
		res: Result=self.__wrapped__.run(command,**dict(dict(warn=True,hide=self.hide),**kwargs))
		if res.failed: raise RuntimeError(res.stderr)
		self.res=res
		return self
	
	def _backup(self,filepath,**kwargs):
		try:
			return self.run(f'cp {filepath} {filepath}_lgblkb_backup',**dict(dict(hide=True),**kwargs))
		except RuntimeError as exc:
			if exc.args[0].rstrip().endswith('Permission denied'):
				return self.run(f'sudo cp {filepath} {filepath}_lgblkb_backup',**kwargs)
			else:
				raise exc
	
	@logger.trace()
	def backup(self,filepath,**kwargs):
		if self.exists_file(filepath+self.backup_tag):
			logger.debug('Backup file already exists. Skipping.')
			return
		else:
			return self._backup(filepath,**kwargs)
	
	def exists_file(self,filepath):
		return bool(int(self.run(f'test -f {filepath} && echo 1 || echo 0',hide=True).res.stdout.strip()))
	
	def exists_folder(self,folderpath):
		return bool(int(self.run(f'test -d {folderpath} && echo 1 || echo 0',hide=True).res.stdout.strip()))
	
	def _cat(self,filepath,file_contents,append):
		file_contents=file_contents.replace('"','""')
		return self.run(f"""sudo bash -c "cat <<EOF {'>>' if append else '>'} {filepath}\n{file_contents}\nEOF" """)
	
	@logger.trace(skimpy=True)
	def create_file(self,filepath,file_contents):
		# file_contents=file_contents.replace('"','""')
		# return self.run(f'sudo bash -c "cat <<EOF > {filepath}\n{file_contents}\nEOF"')
		return self._cat(filepath,file_contents,False)
	
	@logger.trace(skimpy=True)
	def append_to_file(self,filepath,file_contents):
		# file_contents=file_contents.replace('"','""')
		# return self.run(f'sudo bash -c "cat <<EOF >> {filepath}\n{file_contents}\nEOF"')
		return self._cat(filepath,file_contents,True)
	
	@logger.trace(skimpy=True)
	def replace(self,filepath,old,new,delim='/'):
		return self.run(f"""sudo sed -i 's{delim}{old}{delim}{new}{delim}' {filepath}""")

# region RemoteConnection typing fixtures:
RemConn=Union[RemoteConnection,Connection]

def _get_remcon(*args,**kwargs) -> RemConn:
	return Connection(*args,**kwargs)

# endregion


def main():
	conn=RemoteConnection.using(host='10.1.142.180',user='pi')
	conn.hide=False
	# conn.apt_install('dnsmasq')
	# conn.backup('/etc/network/interfaces')
	# 	conn.append_to_file("/etc/network/interfaces",f"""
	# # interfaces(5) file used by ifup(8) and ifdown(8)
	#
	# # Please note that this file is written to be used with dhcpcd
	# # For static IP, consult /etc/dhcpcd.conf and 'man dhcpcd.conf'
	#
	# # Include files from /etc/network/interfaces.d:
	# source-directory /etc/network/interfaces.d
	# allow-hotplug eth0
	# iface eth0 inet static
	#     address 192.168.2.1
	#     netmask 255.255.255.0
	#     network 192.168.2.0
	#     broadcast 192.168.2.255
	# """)
	# 	conn.backup('/etc/dnsmasq.conf')
	# 	conn.create_file('/etc/dnsmasq.conf',"""interface=eth0      # Use interface eth0
	# listen-address=192.168.2.1 # listen on
	# # Bind to the interface to make sure we aren't sending things
	# # elsewhere
	# bind-interfaces
	# server=8.8.8.8       # Forward DNS requests to Google DNS
	# domain-needed        # Don't forward short names
	# # Never forward addresses in the non-routed address spaces.
	# bogus-priv
	# # Assign IP addresses between 192.168.2.2 and 192.168.2.100 with a
	# # 12 hour lease time
	# dhcp-range=192.168.2.2,192.168.2.100,12h""")
	# conn.replace('/etc/sysctl.conf','#net.ipv4.ip_forward=1','net.ipv4.ip_forward=1')
# 	conn.run(f"""
# sudo iptables -t nat -A POSTROUTING -o wlan0 -j MASQUERADE
# sudo iptables -A FORWARD -i wlan0 -o eth0 -m state --state RELATED,ESTABLISHED -j ACCEPT
# sudo iptables -A FORWARD -i eth0 -o wlan0 -j ACCEPT
# """)
# 	conn.run(f"""
# sudo sh -c "iptables-save > /etc/iptables.ipv4.nat"
# """)
# 	conn.backup('/etc/rc.local')
# 	conn.replace('/etc/rc.local','exit 0','iptables-restore < /etc/iptables.ipv4.nat','#').append_to_file('/etc/rc.local','exit 0')
	
	
	pass

if __name__=='__main__':
	main()



import pandas as pd
import pytest

from lgblkb_tools.common import utils
from lgblkb_tools.pathify import *

this_folder_name=os.path.split(os.path.dirname(__file__))[-1]
this_file_name=os.path.split(os.path.splitext(__file__)[0])[-1]

@pytest.fixture()
def this_filepath():
	return __file__

@pytest.fixture()
def this_folder():
	return Folder()

@pytest.fixture()
def some_dirpath():
	return os.path.split(__file__)[0]

def test_get_name(this_filepath):
	name=get_name(this_filepath)
	assert name==this_file_name

def test_get_parent_dir(this_filepath):
	parent_dir=get_parent_dir(this_filepath)
	assert parent_dir==os.path.split(this_filepath)[0]

def test_create_path(this_filepath,some_dirpath):
	created_path=create_path(this_filepath,stop_depth=1)
	assert os.path.exists(created_path)
	assert created_path==this_filepath
	
	created_path=create_path(some_dirpath,stop_depth=0)
	assert os.path.exists(created_path)
	assert created_path==some_dirpath

def test_get_splitted(this_filepath):
	parts=get_splitted(this_filepath)
	assert parts[-1]==this_file_name+'.py'
	assert parts[-2]==this_folder_name

def test_create_zipfile(some_dirpath):
	zipfile_path=create_zipfile(some_dirpath)
	assert get_name(zipfile_path)==this_folder_name
	os.remove(zipfile_path)

def test_folder(this_folder: Folder,this_filepath):
	name_args=[1,2,'some_param']
	assert get_name(this_folder.get_filepath(*name_args,delim='__'))=='1__2__some_param'
	assert this_folder.get_filepath(*name_args,ext='.qwe').endswith('1_2_some_param.qwe')
	assert get_name(this_folder.get_filepath(datetime_loc_index=0)).split('_')[0]==str(datetime.today().date())
	assert this_folder.exists
	assert this_filepath in Folder(__file__).glob_search('*')
	
	tmp_folder=this_folder.create('tmp').clear()
	assert tmp_folder.exists
	assert tmp_folder.create('subdir1').parent().name=='tmp'
	
	tmp2_folder=this_folder.create('tmp2').clear()
	tmp2_folder['qwe.txt']=123
	copied_folder=tmp2_folder.copy_to(tmp_folder)
	assert copied_folder.name==tmp2_folder.name
	assert copied_folder.parent()==tmp_folder
	assert get_name(copied_folder.children[0])=='qwe'
	
	assert tmp2_folder.exists
	tmp2_folder.delete()
	assert not tmp2_folder.exists
	
	src_folder=tmp_folder['tmp_test_src_folder']
	copied_folder.move_to(src_folder)
	assert copied_folder.parent()==src_folder
	
	for i in range(10):
		iter_folder=copied_folder.create('some_folder',iterated=True)
		assert iter_folder.name=='some_folder_'+str(i)
	
	zip_filepath=src_folder.zip()
	assert get_name(zip_filepath)==src_folder.name
	
	safe_folder=tmp_folder.create('qfnowdaofnwoawd.SAFE')
	assert safe_folder.name.endswith('.SAFE')
	for i in range(10):
		safe_folder[f'some_file_{i}.json']=dict(i=1,some_text='fqowaufboadowaodbaw')
	assert len(safe_folder.children)==10
	zip_filepath=safe_folder.zip()
	assert zip_filepath.endswith('.SAFE.zip')
	
	zip_filepath=safe_folder.zip(safe_folder.name.replace('.SAFE',''))
	assert Folder(zip_filepath)==safe_folder.parent()
	assert not zip_filepath.endswith('.SAFE.zip')
	
	src_folder.unzip(zip_filepath,create_subdir=False)
	# assert zip_filepath.rstrip('.zip')==''
	new_safe_folder=src_folder.create(zip_filepath.replace('.zip','.SAFE'),reactive=False)
	assert new_safe_folder.exists
	assert new_safe_folder.name==safe_folder.name
	tmp_folder.delete()
	assert not tmp_folder.exists

def main():
	f1=Folder(r'/home/lgblkb/PycharmProjects/imagination/imagination/data/sentinel2',assert_exists=True)
	f2=Folder(r'/media/lgblkb/9C33-6BBD1/sentinel2',assert_exists=True)
	# utils.run_command(f'cp -ans {f1}* {f2}/')
	# df=pd.DataFrame(map(Folder,[*f1['unzipped'].children,*f2['unzipped'].children]),columns=['safe_folder'])
	
	
	
	
	
		 
	pass
	
	pass

if __name__=='__main__':
	main()



from lgblkb_tools import logger,Folder

# logs_path=Folder('logs')['logs.log']

# logger.add_file_handler(log_folder['logs.log'])
# logger=TheLogger('lgblkb_tools').to_stream().to_rotate(logs_path)

@logger.trace()
def do_something():
	pass

@logger.trace()
def main():
	this_folder=Folder(__file__)
	for child in this_folder.children:
		logger.info('child: %s',child)
	do_something()

	pass

if __name__=='__main__':
	main()



# test_2()
	# test_1()
	# mgr=Manager(host='94.247.135.91',username='docker',password='docker',port='8086')
	# with mgr.session_context():
	# 	a: MainRequest=mgr.session.query(MainRequest).first()
	# 	# print(to_shape(a.cadastre_value))
	# 	print(a.cadastre_value)
	# 	mgr.session.add(SridTest(geom=gmtr.SpatialGeom(to_shape(a.cadastre_value)).convert_crs(4326,3857).geom_obj))
	#
	#
	# return
	# mgr=Manager(host='94.247.135.91',username='docker',password='docker',port='8086')

	# s2_info.priority=136
	# print(s2_info)

	# mgr.s2_info: Sentinel2_Info=s2_info
	# print(mgr.s2_info.priority)
	# mgr.s2_info.priority=136
	# print(mgr.s2_info.priority)
	# print(mgr.s2_info)
	# s2_info=mgr.session.query(Sentinel2_Info).filter(Sentinel2_Info.priority==36).first()